API_BASE_URL: str = "https://api.telegram.org/bot{token}/{method}"
API_FILE_URL: str = "https://api.telegram.org/file/bot{token}/{path}"

DEFAULT_TIMEOUT: float = 30.0
POLLING_TIMEOUT: int = 5
RETRY_ATTEMPTS: int = 3
RETRY_DELAY: float = 1.0
RETRY_BACKOFF: float = 2.0

MAX_MESSAGE_LENGTH: int = 4096
MAX_CAPTION_LENGTH: int = 1024
MAX_CALLBACK_DATA_LENGTH: int = 64

TOKEN_ENV_KEY: str = "BOT_TOKEN"
DEBUG_ENV_KEY: str = "DEBUG"
WEBHOOK_URL_ENV_KEY: str = "WEBHOOK_URL"


class ParseMode:
    HTML = "HTML"
    MARKDOWN = "MarkdownV2"
    MARKDOWN_LEGACY = "Markdown"



from __future__ import annotations
import asyncio
import logging
from collections.abc import Callable
from typing import Any

from gramix.filters import (
    BaseFilter,
    CallbackFilter,
    CommandFilter,
    RegexFilter,
    TextFilter,
)
from gramix.fsm import BaseStorage, MemoryStorage, Step
from gramix.types.callback import CallbackQuery
from gramix.types.chat_member import ChatMemberUpdated
from gramix.types.inline_query import InlineQuery
from gramix.types.message import Message

logger = logging.getLogger(__name__)


class Handler:
    __slots__ = ("func", "filters")

    def __init__(self, func: Callable, filters: list[BaseFilter]) -> None:
        self.func = func
        self.filters = filters

    def matches(self, update: Any) -> bool:
        return all(f.check(update) for f in self.filters)


class Router:
    def __init__(self, storage: BaseStorage | None = None) -> None:
        self._message_handlers: list[Handler] = []
        self._callback_handlers: list[Handler] = []
        self._inline_handlers: list[Handler] = []
        self._chat_member_handlers: list[Callable] = []
        self._state_handlers: dict[str, Callable] = {}
        self.fsm: BaseStorage = storage if storage is not None else MemoryStorage()

    def message(self, *args: str | BaseFilter, **kwargs: Any) -> Callable:
        filters = self._build_message_filters(args, kwargs)

        def decorator(func: Callable) -> Callable:
            self._message_handlers.append(Handler(func, filters))
            logger.debug("Handler: %s → %s", func.__name__, [type(f).__name__ for f in filters])
            return func

        return decorator

    def callback(self, *data: str, prefix: str | None = None) -> Callable:
        from gramix.filters import CallbackPrefixFilter

        filters: list[BaseFilter] = (
            [CallbackPrefixFilter(prefix)] if prefix
            else ([CallbackFilter(*data)] if data else [])
        )

        def decorator(func: Callable) -> Callable:
            self._callback_handlers.append(Handler(func, filters))
            return func

        return decorator

    def inline(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            self._inline_handlers.append(Handler(func, []))
            return func
        return decorator

    def chat_member(self) -> Callable:
        def decorator(func: Callable) -> Callable:
            self._chat_member_handlers.append(func)
            return func
        return decorator

    def state(self, step: Step) -> Callable:
        key = f"{step._owner}.{step._name}"

        def decorator(func: Callable) -> Callable:
            self._state_handlers[key] = func
            logger.debug("FSM handler: %s → %s", key, func.__name__)
            return func

        return decorator

    def process_message(self, message: Message) -> bool:
        user_id = message.from_user.id if message.from_user else None

        if user_id:
            ctx = self.fsm.get(user_id)
            if ctx.is_active and ctx.current in self._state_handlers:
                self._call(self._state_handlers[ctx.current], message, ctx)
                return True

        for handler in self._message_handlers:
            if handler.matches(message):
                self._call(handler.func, message)
                return True

        return False

    async def async_process_message(self, message: Message) -> bool:
        user_id = message.from_user.id if message.from_user else None

        if user_id:
            ctx = self.fsm.get(user_id)
            if ctx.is_active and ctx.current in self._state_handlers:
                await self._async_call(self._state_handlers[ctx.current], message, ctx)
                return True

        for handler in self._message_handlers:
            if handler.matches(message):
                await self._async_call(handler.func, message)
                return True

        return False

    def process_callback(self, callback: CallbackQuery) -> bool:
        for handler in self._callback_handlers:
            if handler.matches(callback):
                self._call(handler.func, callback)
                return True
        return False

    async def async_process_callback(self, callback: CallbackQuery) -> bool:
        for handler in self._callback_handlers:
            if handler.matches(callback):
                await self._async_call(handler.func, callback)
                return True
        return False

    def process_inline(self, query: InlineQuery) -> bool:
        for handler in self._inline_handlers:
            self._call(handler.func, query)
            return True
        return False

    async def async_process_inline(self, query: InlineQuery) -> bool:
        for handler in self._inline_handlers:
            await self._async_call(handler.func, query)
            return True
        return False

    def process_chat_member(self, update: ChatMemberUpdated) -> bool:
        for func in self._chat_member_handlers:
            self._call(func, update)
        return bool(self._chat_member_handlers)

    async def async_process_chat_member(self, update: ChatMemberUpdated) -> bool:
        for func in self._chat_member_handlers:
            await self._async_call(func, update)
        return bool(self._chat_member_handlers)

    def _call(self, func: Callable, *args: Any) -> None:
        try:
            func(*args)
        except Exception:
            logger.exception("Ошибка в handler %s:", func.__name__)

    async def _async_call(self, func: Callable, *args: Any) -> None:
        try:
            if asyncio.iscoroutinefunction(func):
                await func(*args)
            else:
                func(*args)
        except Exception:
            logger.exception("Ошибка в handler %s:", func.__name__)

    def _build_message_filters(self, args: tuple, kwargs: dict) -> list[BaseFilter]:
        filters: list[BaseFilter] = []
        for arg in args:
            if isinstance(arg, str):
                filters.append(CommandFilter(arg) if arg.startswith("/") else TextFilter(arg))
            elif isinstance(arg, BaseFilter):
                filters.append(arg)

        if "regex" in kwargs:
            filters.append(RegexFilter(kwargs["regex"]))
        if "text" in kwargs:
            filters.append(TextFilter(kwargs["text"]))
        if "command" in kwargs:
            filters.append(CommandFilter(kwargs["command"]))

        return filters



from __future__ import annotations
from dataclasses import dataclass


@dataclass(frozen=True, slots=True)
class User:
    id: int
    is_bot: bool
    first_name: str
    last_name: str | None = None
    username: str | None = None
    language_code: str | None = None
    is_premium: bool = False

    @property
    def full_name(self) -> str:
        if self.last_name:
            return f"{self.first_name} {self.last_name}"
        return self.first_name

    @property
    def mention(self) -> str:
        if self.username:
            return f"@{self.username}"
        return f'<a href="tg://user?id={self.id}">{self.full_name}</a>'

    @classmethod
    def from_dict(cls, data: dict) -> User:
        return cls(
            id=data["id"],
            is_bot=data.get("is_bot", False),
            first_name=data["first_name"],
            last_name=data.get("last_name"),
            username=data.get("username"),
            language_code=data.get("language_code"),
            is_premium=data.get("is_premium", False),
        )



from __future__ import annotations
from typing import TYPE_CHECKING

from gramix.types.user import User
from gramix.types.message import Message

if TYPE_CHECKING:
    from gramix.bot import Bot


class CallbackQuery:
    __slots__ = ("id", "from_user", "message", "data", "inline_message_id", "_bot")

    def __init__(
        self,
        id: str,
        from_user: User,
        message: Message | None,
        data: str | None,
        inline_message_id: str | None,
        bot: Bot,
    ) -> None:
        self.id = id
        self.from_user = from_user
        self.message = message
        self.data = data
        self.inline_message_id = inline_message_id
        self._bot = bot

    def answer(
        self,
        text: str | None = None,
        *,
        show_alert: bool = False,
        url: str | None = None,
        cache_time: int = 0,
    ) -> bool:
        return self._bot.answer_callback_query(
            callback_query_id=self.id,
            text=text,
            show_alert=show_alert,
            url=url,
            cache_time=cache_time,
        )

    @classmethod
    def from_dict(cls, data: dict, bot: Bot) -> CallbackQuery:
        return cls(
            id=data["id"],
            from_user=User.from_dict(data["from"]),
            message=Message.from_dict(data["message"], bot) if "message" in data else None,
            data=data.get("data"),
            inline_message_id=data.get("inline_message_id"),
            bot=bot,
        )



from __future__ import annotations
import asyncio
import logging
from collections.abc import Callable
from typing import Any

logger = logging.getLogger(__name__)


class MiddlewareManager:
    def __init__(self) -> None:
        self._middlewares: list[Callable] = []

    def register(self, func: Callable) -> Callable:
        self._middlewares.append(func)
        logger.debug("Middleware зарегистрирован: %s", func.__name__)
        return func

    def run(self, update: Any, handler: Callable) -> None:
        chain = list(self._middlewares)

        def call_next(idx: int) -> None:
            if idx < len(chain):
                chain[idx](update, lambda: call_next(idx + 1))
            else:
                handler(update)

        call_next(0)

    async def async_run(self, update: Any, handler: Callable) -> None:
        chain = list(self._middlewares)

        async def call_next(idx: int) -> None:
            if idx < len(chain):
                mw = chain[idx]

                async def next_fn() -> None:
                    await call_next(idx + 1)

                if asyncio.iscoroutinefunction(mw):
                    await mw(update, next_fn)
                else:
                    mw(update, next_fn)
            else:
                if asyncio.iscoroutinefunction(handler):
                    await handler(update)
                else:
                    handler(update)

        await call_next(0)



from __future__ import annotations
from dataclasses import dataclass

from gramix.constants import MAX_CALLBACK_DATA_LENGTH
from gramix.exceptions import FilterError


@dataclass(slots=True)
class InlineButton:
    text: str
    callback_data: str | None = None
    url: str | None = None
    switch_inline_query: str | None = None
    switch_inline_query_current_chat: str | None = None

    def to_dict(self) -> dict:
        btn: dict = {"text": self.text}
        if self.callback_data is not None:
            if len(self.callback_data) > MAX_CALLBACK_DATA_LENGTH:
                raise FilterError(
                    f"callback_data не должен превышать {MAX_CALLBACK_DATA_LENGTH} символов."
                )
            btn["callback_data"] = self.callback_data
        elif self.url is not None:
            btn["url"] = self.url
        elif self.switch_inline_query is not None:
            btn["switch_inline_query"] = self.switch_inline_query
        elif self.switch_inline_query_current_chat is not None:
            btn["switch_inline_query_current_chat"] = self.switch_inline_query_current_chat
        return btn


class Inline:
    def __init__(self) -> None:
        self._rows: list[list[InlineButton]] = [[]]

    def button(
        self,
        text: str,
        *,
        callback: str | None = None,
        url: str | None = None,
        switch_inline: str | None = None,
        switch_inline_current: str | None = None,
    ) -> Inline:
        self._rows[-1].append(
            InlineButton(
                text=text,
                callback_data=callback,
                url=url,
                switch_inline_query=switch_inline,
                switch_inline_query_current_chat=switch_inline_current,
            )
        )
        return self

    def row(self) -> Inline:
        if self._rows[-1]:
            self._rows.append([])
        return self

    def to_dict(self) -> dict:
        rows = [row for row in self._rows if row]
        return {"inline_keyboard": [[btn.to_dict() for btn in row] for row in rows]}


@dataclass(slots=True)
class ReplyButton:
    text: str
    request_contact: bool = False
    request_location: bool = False

    def to_dict(self) -> dict:
        btn: dict = {"text": self.text}
        if self.request_contact:
            btn["request_contact"] = True
        if self.request_location:
            btn["request_location"] = True
        return btn


class Reply:
    def __init__(
        self,
        *,
        resize: bool = True,
        one_time: bool = False,
        placeholder: str | None = None,
    ) -> None:
        self._rows: list[list[ReplyButton]] = [[]]
        self._resize = resize
        self._one_time = one_time
        self._placeholder = placeholder

    def button(self, text: str, *, contact: bool = False, location: bool = False) -> Reply:
        self._rows[-1].append(
            ReplyButton(text=text, request_contact=contact, request_location=location)
        )
        return self

    def row(self) -> Reply:
        if self._rows[-1]:
            self._rows.append([])
        return self

    def to_dict(self) -> dict:
        rows = [row for row in self._rows if row]
        result: dict = {
            "keyboard": [[btn.to_dict() for btn in row] for row in rows],
            "resize_keyboard": self._resize,
            "one_time_keyboard": self._one_time,
        }
        if self._placeholder:
            result["input_field_placeholder"] = self._placeholder
        return result


class RemoveKeyboard:
    def to_dict(self) -> dict:
        return {"remove_keyboard": True}



from __future__ import annotations
from dataclasses import dataclass

from gramix.types.user import User
from gramix.types.chat import Chat


@dataclass(slots=True)
class ChatMemberUpdated:
    chat: Chat
    from_user: User
    date: int
    old_status: str
    new_status: str
    user: User

    @property
    def joined(self) -> bool:
        return self.new_status in ("member", "administrator", "creator") and \
               self.old_status in ("left", "kicked", "restricted")

    @property
    def left(self) -> bool:
        return self.new_status in ("left", "kicked") and \
               self.old_status in ("member", "administrator", "creator", "restricted")

    @classmethod
    def from_dict(cls, data: dict) -> ChatMemberUpdated:
        old_member = data.get("old_chat_member", {})
        new_member = data.get("new_chat_member", {})
        return cls(
            chat=Chat.from_dict(data["chat"]),
            from_user=User.from_dict(data["from"]),
            date=data["date"],
            old_status=old_member.get("status", "left"),
            new_status=new_member.get("status", "left"),
            user=User.from_dict(new_member.get("user", old_member.get("user", {}))),
        )



from gramix.types.user import User
from gramix.types.chat import Chat, ChatType
from gramix.types.message import Message, PhotoSize, Document, Audio, Video, Voice, Sticker
from gramix.types.callback import CallbackQuery
from gramix.types.keyboard import Inline, Reply, RemoveKeyboard
from gramix.types.inline_query import InlineQuery, InlineQueryResultArticle
from gramix.types.chat_member import ChatMemberUpdated

__all__ = [
    "User",
    "Chat",
    "ChatType",
    "Message",
    "PhotoSize",
    "Document",
    "Audio",
    "Video",
    "Voice",
    "Sticker",
    "CallbackQuery",
    "Inline",
    "Reply",
    "RemoveKeyboard",
    "InlineQuery",
    "InlineQueryResultArticle",
    "ChatMemberUpdated",
]



from __future__ import annotations
from dataclasses import dataclass
from typing import TYPE_CHECKING

from gramix.types.user import User
from gramix.types.chat import Chat
from gramix.types.keyboard import Inline, Reply, RemoveKeyboard
from gramix.constants import MAX_MESSAGE_LENGTH, MAX_CAPTION_LENGTH
from gramix.exceptions import MessageError

if TYPE_CHECKING:
    from gramix.bot import Bot


@dataclass(slots=True)
class PhotoSize:
    file_id: str
    file_unique_id: str
    width: int
    height: int
    file_size: int | None = None

    @classmethod
    def from_dict(cls, data: dict) -> PhotoSize:
        return cls(
            file_id=data["file_id"],
            file_unique_id=data["file_unique_id"],
            width=data["width"],
            height=data["height"],
            file_size=data.get("file_size"),
        )


@dataclass(slots=True)
class Document:
    file_id: str
    file_unique_id: str
    file_name: str | None = None
    mime_type: str | None = None
    file_size: int | None = None

    @classmethod
    def from_dict(cls, data: dict) -> Document:
        return cls(
            file_id=data["file_id"],
            file_unique_id=data["file_unique_id"],
            file_name=data.get("file_name"),
            mime_type=data.get("mime_type"),
            file_size=data.get("file_size"),
        )


@dataclass(slots=True)
class Audio:
    file_id: str
    file_unique_id: str
    duration: int
    performer: str | None = None
    title: str | None = None
    file_name: str | None = None
    mime_type: str | None = None
    file_size: int | None = None

    @classmethod
    def from_dict(cls, data: dict) -> Audio:
        return cls(
            file_id=data["file_id"],
            file_unique_id=data["file_unique_id"],
            duration=data["duration"],
            performer=data.get("performer"),
            title=data.get("title"),
            file_name=data.get("file_name"),
            mime_type=data.get("mime_type"),
            file_size=data.get("file_size"),
        )


@dataclass(slots=True)
class Video:
    file_id: str
    file_unique_id: str
    width: int
    height: int
    duration: int
    file_name: str | None = None
    mime_type: str | None = None
    file_size: int | None = None

    @classmethod
    def from_dict(cls, data: dict) -> Video:
        return cls(
            file_id=data["file_id"],
            file_unique_id=data["file_unique_id"],
            width=data["width"],
            height=data["height"],
            duration=data["duration"],
            file_name=data.get("file_name"),
            mime_type=data.get("mime_type"),
            file_size=data.get("file_size"),
        )


@dataclass(slots=True)
class Voice:
    file_id: str
    file_unique_id: str
    duration: int
    mime_type: str | None = None
    file_size: int | None = None

    @classmethod
    def from_dict(cls, data: dict) -> Voice:
        return cls(
            file_id=data["file_id"],
            file_unique_id=data["file_unique_id"],
            duration=data["duration"],
            mime_type=data.get("mime_type"),
            file_size=data.get("file_size"),
        )


@dataclass(slots=True)
class Sticker:
    file_id: str
    file_unique_id: str
    width: int
    height: int
    is_animated: bool = False
    is_video: bool = False
    emoji: str | None = None
    file_size: int | None = None

    @classmethod
    def from_dict(cls, data: dict) -> Sticker:
        return cls(
            file_id=data["file_id"],
            file_unique_id=data["file_unique_id"],
            width=data["width"],
            height=data["height"],
            is_animated=data.get("is_animated", False),
            is_video=data.get("is_video", False),
            emoji=data.get("emoji"),
            file_size=data.get("file_size"),
        )


class Message:
    __slots__ = (
        "message_id", "date", "chat", "from_user",
        "text", "photo", "document", "audio", "video",
        "voice", "sticker", "caption", "reply_to_message",
        "forward_from", "forward_date", "_bot",
    )

    def __init__(
        self,
        message_id: int,
        date: int,
        chat: Chat,
        from_user: User | None,
        text: str | None,
        photo: list[PhotoSize] | None,
        document: Document | None,
        audio: Audio | None,
        video: Video | None,
        voice: Voice | None,
        sticker: Sticker | None,
        caption: str | None,
        reply_to_message: Message | None,
        forward_from: User | None,
        forward_date: int | None,
        bot: Bot,
    ) -> None:
        self.message_id = message_id
        self.date = date
        self.chat = chat
        self.from_user = from_user
        self.text = text
        self.photo = photo
        self.document = document
        self.audio = audio
        self.video = video
        self.voice = voice
        self.sticker = sticker
        self.caption = caption
        self.reply_to_message = reply_to_message
        self.forward_from = forward_from
        self.forward_date = forward_date
        self._bot = bot

    @property
    def content(self) -> str | None:
        return self.text or self.caption

    def reply(
        self,
        text: str,
        *,
        keyboard: Inline | Reply | RemoveKeyboard | None = None,
        parse_mode: str | None = None,
        disable_preview: bool = False,
    ) -> Message:
        if len(text) > MAX_MESSAGE_LENGTH:
            raise MessageError(f"Текст превышает {MAX_MESSAGE_LENGTH} символов.")
        return self._bot.send_message(
            chat_id=self.chat.id,
            text=text,
            reply_to_message_id=self.message_id,
            keyboard=keyboard,
            parse_mode=parse_mode,
            disable_preview=disable_preview,
        )

    def answer(
        self,
        text: str,
        *,
        keyboard: Inline | Reply | RemoveKeyboard | None = None,
        parse_mode: str | None = None,
        disable_preview: bool = False,
    ) -> Message:
        if len(text) > MAX_MESSAGE_LENGTH:
            raise MessageError(f"Текст превышает {MAX_MESSAGE_LENGTH} символов.")
        return self._bot.send_message(
            chat_id=self.chat.id,
            text=text,
            keyboard=keyboard,
            parse_mode=parse_mode,
            disable_preview=disable_preview,
        )

    def edit(
        self,
        text: str,
        *,
        keyboard: Inline | None = None,
        parse_mode: str | None = None,
    ) -> Message:
        return self._bot.edit_message_text(
            chat_id=self.chat.id,
            message_id=self.message_id,
            text=text,
            keyboard=keyboard,
            parse_mode=parse_mode,
        )

    def delete(self) -> bool:
        return self._bot.delete_message(chat_id=self.chat.id, message_id=self.message_id)

    def pin(self, *, disable_notification: bool = False) -> bool:
        return self._bot.pin_chat_message(
            chat_id=self.chat.id,
            message_id=self.message_id,
            disable_notification=disable_notification,
        )

    def forward(self, to_chat_id: int | str) -> Message:
        return self._bot.forward_message(
            chat_id=to_chat_id,
            from_chat_id=self.chat.id,
            message_id=self.message_id,
        )

    def copy(self, to_chat_id: int | str, *, caption: str | None = None) -> int:
        return self._bot.copy_message(
            chat_id=to_chat_id,
            from_chat_id=self.chat.id,
            message_id=self.message_id,
            caption=caption,
        )

    def reply_photo(
        self,
        photo: str,
        *,
        caption: str | None = None,
        keyboard: Inline | Reply | None = None,
        parse_mode: str | None = None,
    ) -> Message:
        if caption and len(caption) > MAX_CAPTION_LENGTH:
            raise MessageError(f"Подпись превышает {MAX_CAPTION_LENGTH} символов.")
        return self._bot.send_photo(
            chat_id=self.chat.id,
            photo=photo,
            caption=caption,
            keyboard=keyboard,
            parse_mode=parse_mode,
        )

    def reply_document(
        self,
        document: str,
        *,
        caption: str | None = None,
        keyboard: Inline | Reply | None = None,
    ) -> Message:
        return self._bot.send_document(
            chat_id=self.chat.id,
            document=document,
            caption=caption,
            keyboard=keyboard,
        )

    def reply_video(
        self,
        video: str,
        *,
        caption: str | None = None,
        keyboard: Inline | Reply | None = None,
        parse_mode: str | None = None,
    ) -> Message:
        return self._bot.send_video(
            chat_id=self.chat.id,
            video=video,
            caption=caption,
            keyboard=keyboard,
            parse_mode=parse_mode,
        )

    def reply_audio(
        self,
        audio: str,
        *,
        caption: str | None = None,
        keyboard: Inline | Reply | None = None,
    ) -> Message:
        return self._bot.send_audio(
            chat_id=self.chat.id,
            audio=audio,
            caption=caption,
            keyboard=keyboard,
        )

    def reply_voice(self, voice: str, *, caption: str | None = None) -> Message:
        return self._bot.send_voice(
            chat_id=self.chat.id,
            voice=voice,
            caption=caption,
        )

    def react(self, emoji: str) -> bool:
        return self._bot.set_message_reaction(
            chat_id=self.chat.id,
            message_id=self.message_id,
            reaction=emoji,
        )

    @classmethod
    def from_dict(cls, data: dict, bot: Bot) -> Message:
        photo = (
            [PhotoSize.from_dict(p) for p in data["photo"]] if "photo" in data else None
        )
        reply = (
            Message.from_dict(data["reply_to_message"], bot)
            if "reply_to_message" in data
            else None
        )
        return cls(
            message_id=data["message_id"],
            date=data["date"],
            chat=Chat.from_dict(data["chat"]),
            from_user=User.from_dict(data["from"]) if "from" in data else None,
            text=data.get("text"),
            photo=photo,
            document=Document.from_dict(data["document"]) if "document" in data else None,
            audio=Audio.from_dict(data["audio"]) if "audio" in data else None,
            video=Video.from_dict(data["video"]) if "video" in data else None,
            voice=Voice.from_dict(data["voice"]) if "voice" in data else None,
            sticker=Sticker.from_dict(data["sticker"]) if "sticker" in data else None,
            caption=data.get("caption"),
            reply_to_message=reply,
            forward_from=User.from_dict(data["forward_from"]) if "forward_from" in data else None,
            forward_date=data.get("forward_date"),
            bot=bot,
        )



from __future__ import annotations
from dataclasses import dataclass
from enum import StrEnum


class ChatType(StrEnum):
    PRIVATE = "private"
    GROUP = "group"
    SUPERGROUP = "supergroup"
    CHANNEL = "channel"


@dataclass(frozen=True, slots=True)
class Chat:
    id: int
    type: ChatType
    title: str | None = None
    username: str | None = None
    first_name: str | None = None
    last_name: str | None = None

    @property
    def is_private(self) -> bool:
        return self.type == ChatType.PRIVATE

    @property
    def is_group(self) -> bool:
        return self.type in (ChatType.GROUP, ChatType.SUPERGROUP)

    @property
    def is_channel(self) -> bool:
        return self.type == ChatType.CHANNEL

    @property
    def display_name(self) -> str:
        if self.title:
            return self.title
        if self.first_name:
            parts = [self.first_name]
            if self.last_name:
                parts.append(self.last_name)
            return " ".join(parts)
        return str(self.id)

    @classmethod
    def from_dict(cls, data: dict) -> Chat:
        return cls(
            id=data["id"],
            type=ChatType(data["type"]),
            title=data.get("title"),
            username=data.get("username"),
            first_name=data.get("first_name"),
            last_name=data.get("last_name"),
        )



class GramixError(Exception):
    pass


class TokenError(GramixError):
    pass


class TelegramAPIError(GramixError):
    def __init__(self, code: int, description: str) -> None:
        self.code = code
        self.description = description
        super().__init__(f"Telegram API error {code}: {description}")


class NetworkError(GramixError):
    pass


class RetryAfterError(TelegramAPIError):
    def __init__(self, retry_after: int) -> None:
        self.retry_after = retry_after
        super().__init__(429, f"Flood control. Retry after {retry_after}s.")


class MessageError(GramixError):
    pass


class FSMError(GramixError):
    pass


class FilterError(GramixError):
    pass


class MiddlewareError(GramixError):
    pass


class WebhookError(GramixError):
    pass


class FileError(GramixError):
    pass



from __future__ import annotations
import re
from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from gramix.types.message import Message
    from gramix.types.callback import CallbackQuery


class BaseFilter(ABC):
    @abstractmethod
    def check(self, update: object) -> bool:
        pass


class CommandFilter(BaseFilter):
    def __init__(self, *commands: str) -> None:
        self._commands = frozenset(cmd.lstrip("/").lower() for cmd in commands)

    def check(self, update: Message) -> bool:
        if not update.text or not update.text.startswith("/"):
            return False
        command = update.text.split()[0].lstrip("/").split("@")[0].lower()
        return command in self._commands


class TextFilter(BaseFilter):
    def __init__(self, *texts: str, case_sensitive: bool = False) -> None:
        self._case_sensitive = case_sensitive
        self._texts = frozenset(texts if case_sensitive else (t.lower() for t in texts))

    def check(self, update: Message) -> bool:
        if not update.text:
            return False
        text = update.text if self._case_sensitive else update.text.lower()
        return text in self._texts


class RegexFilter(BaseFilter):
    def __init__(self, pattern: str, flags: int = 0) -> None:
        self._pattern = re.compile(pattern, flags)

    def check(self, update: Message) -> bool:
        return bool(update.text and self._pattern.search(update.text))


class CallbackFilter(BaseFilter):
    def __init__(self, *data: str) -> None:
        self._data = frozenset(data)

    def check(self, update: CallbackQuery) -> bool:
        return update.data in self._data


class CallbackPrefixFilter(BaseFilter):
    def __init__(self, *prefixes: str) -> None:
        self._prefixes = tuple(prefixes)

    def check(self, update: CallbackQuery) -> bool:
        return bool(update.data and update.data.startswith(self._prefixes))


class HasPhotoFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.photo is not None


class HasDocumentFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.document is not None


class HasVideoFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.video is not None


class HasAudioFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.audio is not None


class HasVoiceFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.voice is not None


class HasStickerFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.sticker is not None


class HasTextFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return bool(update.text)


class IsForwardFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.forward_from is not None


class IsReplyFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.reply_to_message is not None


class ChatTypeFilter(BaseFilter):
    def __init__(self, *types: str) -> None:
        self._types = frozenset(types)

    def check(self, update: Message) -> bool:
        return update.chat.type.value in self._types


class PrivateChatFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.chat.is_private


class GroupChatFilter(BaseFilter):
    def check(self, update: Message) -> bool:
        return update.chat.is_group


class F:
    photo = HasPhotoFilter()
    document = HasDocumentFilter()
    video = HasVideoFilter()
    audio = HasAudioFilter()
    voice = HasVoiceFilter()
    sticker = HasStickerFilter()
    text = HasTextFilter()
    forward = IsForwardFilter()
    reply = IsReplyFilter()
    private = PrivateChatFilter()
    group = GroupChatFilter()
